@@ -5,8 +5,7 @@ loadModule("xmap_module", TRUE)
 .onAttach <- function(...) {
     if (!interactive()) return()
     
-    intro_message <- paste("If you're new to the rEDM package, please check out 
-                           the tutorial:",
+    intro_message <- paste("If you're new to the rEDM package, please check out the tutorial:",
                            "> vignette(\"rEDM-tutorial\")", sep = "\n")
     packageStartupMessage(intro_message)
 }
\ No newline at end of file
