@@ -5,6 +5,7 @@ const double ForecastMachine::qnan = std::numeric_limits<double>::quiet_NaN();
 
 ForecastMachine::ForecastMachine():
 lib_indices(std::vector<bool>()), pred_indices(std::vector<bool>()),
+pred_requested_indices(std::vector<bool>()), 
 which_lib(std::vector<size_t>()), which_pred(std::vector<size_t>()),
 time(vec()), data_vectors(std::vector<vec>()), smap_coefficients(std::vector<vec>()),
 targets(vec()), predicted(vec()), predicted_var(vec()),
@@ -309,6 +310,47 @@ void ForecastMachine::set_indices_from_range(std::vector<bool>& indices, const s
 	return;
 }
 
+void ForecastMachine::set_pred_requested_indices_from_range(std::vector<bool>& indices, 
+                                             const std::vector<time_range> range)
+{
+    size_t start_of_range, end_of_range;
+    indices.assign(num_vectors, false); // initialize indices
+    for(auto& range_iter: range)
+    {
+        start_of_range = range_iter.first;
+        if(start_of_range >= num_vectors) // check start of range
+        {
+            std::ostringstream temp;
+            temp << "start_of_range = ";
+            temp << start_of_range + 1;
+            temp << ", but num_vectors = ";
+            temp << num_vectors;
+            std::string temp_str = temp.str();
+            LOG_WARNING(temp_str.c_str());
+            LOG_WARNING("start of time_range was greater than the number of vectors; skipping");
+            continue;
+        }
+        
+        end_of_range = range_iter.second;
+        if(end_of_range >= num_vectors) // check end of range
+        {
+            std::ostringstream temp;
+            temp << "end_of_range = ";
+            temp << end_of_range + 1;
+            temp << ", but num_vectors = ";
+            temp << num_vectors;
+            std::string temp_str = temp.str();
+            LOG_WARNING(temp_str.c_str());
+            LOG_WARNING("end of time_range was greater than the number of vectors; corrected");
+            end_of_range = num_vectors-1;
+        }
+        for(size_t j = start_of_range; j <= end_of_range; ++j)
+        {
+            indices[j] = true;
+        }
+    }
+    return;
+}
 void ForecastMachine::check_cross_validation()
 {
     if (exclusion_radius >= 0) // if exclusion_radius is set, always do cross_validation
@@ -428,6 +470,8 @@ void ForecastMachine::smap_forecast()
     */
     if(SAVE_SMAP_COEFFICIENTS)
     {
+        List tmp(num_vectors);
+        smap_coefficient_covariances = tmp;
         smap_coefficients.assign(data_vectors[0].size()+1, vec(num_vectors, qnan));
     }
     smap_prediction(0, which_pred.size());
@@ -522,8 +566,8 @@ void ForecastMachine::simplex_prediction(const size_t start, const size_t end)
         for(size_t k = 0; k < effective_nn; ++k)
             predicted_var[curr_pred] += weights[k] * pow(targets[nearest_neighbors[k]] - predicted[curr_pred], 2);
         predicted_var[curr_pred] = predicted_var[curr_pred] / total_weight;
-        if(predicted_var[curr_pred] == 0)
-            LOG_WARNING("Zero prediction uncertainty.");
+//        if(predicted_var[curr_pred] == 0)
+//            LOG_WARNING("Zero prediction uncertainty.");
     }
     return;
 }
@@ -563,8 +607,7 @@ void ForecastMachine::smap_prediction(const size_t start, const size_t end)
             LOG_WARNING("no nearest neighbors found; using NA for forecast");
             continue;
         }
-        weights = Eigen::VectorXd::Constant(effective_nn, 1.0);
-        //        weights.assign(effective_nn, 1.0); // default is for theta = 0
+        weights = Eigen::VectorXd::Constant(effective_nn, 1.0); // default is for theta = 0
         if(theta > 0.0)
         {
             // compute average distance
@@ -617,20 +660,22 @@ void ForecastMachine::smap_prediction(const size_t start, const size_t end)
         {
             for(size_t j = 0; j <= E; ++j)
                 smap_coefficients[j][curr_pred] = x(j);
+            
+            // compute covariance matrix for coefficients
+            MatrixXd H = svd.matrixV() * S_inv * svd.matrixU().transpose() * weights.asDiagonal();
+            
+            VectorXd w_resid = B - A * x;
+            double total_w = 0;
+            for(size_t i = 0; i < effective_nn; ++i)
+            {
+                total_w += weights(i) * weights(i);
+            }
+            double sigma_squared = w_resid.dot(w_resid) / total_w;
+            smap_coefficient_covariances[curr_pred] = sigma_squared * H * H.transpose();
         }
         // save prediction
         predicted[curr_pred] = pred;
         
-        // compute variance of prediction through residuals of fitted s-map model
-        /*
-        VectorXd w_resid = B - A * x;
-        double total_w = 0;
-        for(size_t i = 0; i < effective_nn; ++i)
-        {
-            total_w += weights(i) * weights(i);
-        }
-        predicted_var[curr_pred] = w_resid.dot(w_resid) / total_w;
-        */
         // compute variance of prediction (using same approach as simplex)
         predicted_var[curr_pred] = 0;
         double total_weight = 0;
@@ -640,8 +685,8 @@ void ForecastMachine::smap_prediction(const size_t start, const size_t end)
             predicted_var[curr_pred] += weights(k) * pow(targets[nearest_neighbors[k]] - predicted[curr_pred], 2);
         }
         predicted_var[curr_pred] = predicted_var[curr_pred] / total_weight;
-        if(predicted_var[curr_pred] == 0)
-            LOG_WARNING("Zero prediction uncertainty.");
+//        if(predicted_var[curr_pred] == 0)
+//            LOG_WARNING("Zero prediction uncertainty.");
     }
     return;
 }
